import 'package:flutter/cupertino.dart';
import 'package:flutter/material.dart';
import 'package:intl/intl.dart';
import 'package:provider/provider.dart';
import 'package:retgoo_sdk/models/notification.dart';
import 'package:retgoo_sdk/pages/process/process_page.dart';
import 'package:retgoo_sdk/providers/notification_provider.dart';
import 'package:retgoo_sdk/retgoo_sdk.dart';
import 'package:timeago/timeago.dart' as timeago;

enum _SortMode { asc, desc }

class NotificationPage extends StatefulWidget {
  const NotificationPage({Key key}) : super(key: key);

  @override
  _NotificationPageState createState() => _NotificationPageState();
}

class _NotificationPageState extends State<NotificationPage>
    with TickerProviderStateMixin {
  _SortMode _sortMode = _SortMode.desc;
  String filter = "";
  TabController tabController;

  @override
  void initState() {
    super.initState();
    tabController = TabController(
      length: 2,
      vsync: this,
    );

    tabController.addListener(() {
      setState(() {});
    });
  }

  @override
  void dispose() {
    tabController.dispose();
    super.dispose();
  }

  @override
  Widget build(BuildContext context) {
    return Selector<NotificationProvider, NotificationList>(
      selector: (context, provider) => provider.list,
      builder: (context, list, child) {
        if (list == null) {
          return Center(
            child: CircularProgressIndicator(),
          );
        }

        if (_sortMode == _SortMode.asc) {
          list.todoList.sort(
              (a, b) => a.processDateCreated.compareTo(b.processDateCreated));
        } else {
          list.todoList.sort(
              (b, a) => a.processDateCreated.compareTo(b.processDateCreated));
        }

        final todoList = getActivities(list);

        return Column(
          children: [
            const SizedBox(
              height: 16.0,
            ),
            Padding(
              padding: const EdgeInsets.symmetric(
                vertical: 8.0,
                horizontal: 16.0,
              ),
              child: Column(
                crossAxisAlignment: CrossAxisAlignment.stretch,
                children: [
                  Row(
                    children: [
                      Expanded(
                        child: Column(
                          crossAxisAlignment: CrossAxisAlignment.start,
                          children: [
                            Text(
                              "home.tasks".tr,
                              style: TextStyle(
                                fontSize: 24.0,
                                fontWeight: FontWeight.w900,
                              ),
                            ),
                            Text(
                              "home.tasks_description".tr,
                              style: TextStyle(),
                            ),
                          ],
                        ),
                      ),
                      const SizedBox(width: 8.0),
                      Builder(builder: (context) {
                        return ToggleButtons(
                          constraints: BoxConstraints(
                            minWidth: 100,
                            minHeight: 36,
                          ),
                          fillColor: Theme.of(context).primaryColor,
                          borderColor: Theme.of(context).primaryColor,
                          selectedBorderColor: Theme.of(context).primaryColor,
                          selectedColor: Colors.white,
                          borderRadius: BorderRadius.circular(8.0),
                          isSelected: [
                            tabController.index == 0,
                            tabController.index == 1,
                          ],
                          children: [
                            Text("tab.todo".tr),
                            Text("tab.outstanding".tr),
                          ],
                          onPressed: (index) {
                            setState(() {
                              tabController.index = index;
                            });
                          },
                        );
                      }),
                    ],
                  ),
                  const SizedBox(height: 8.0),
                  Row(
                    children: [
                      Expanded(
                        child: Card(
                          margin: const EdgeInsets.all(0.0),
                          child: TextField(
                            onChanged: (value) {
                              setState(() {
                                filter = value;
                              });
                            },
                            decoration: InputDecoration(
                              border: OutlineInputBorder(
                                  borderSide: BorderSide.none),
                              contentPadding: const EdgeInsets.symmetric(
                                horizontal: 16.0,
                                vertical: 10.0,
                              ),
                              prefixIcon: Icon(Icons.search),
                              hintText: "lbl.find_caption".tr,
                            ),
                          ),
                        ),
                      ),
                      IconButton(
                        icon: Icon(_sortMode == _SortMode.asc
                            ? CupertinoIcons.sort_up
                            : CupertinoIcons.sort_down),
                        onPressed: () {
                          setState(() {
                            _sortMode = _sortMode == _SortMode.asc
                                ? _SortMode.desc
                                : _SortMode.asc;
                          });
                        },
                      ),
                    ],
                  ),
                ],
              ),
            ),
            Expanded(
              child: RefreshIndicator(
                notificationPredicate: (notification) {
                  return true;
                },
                onRefresh: () async {
                  await Provider.of<NotificationProvider>(context,
                          listen: false)
                      .refresh();
                },
                child: TabBarView(
                  controller: tabController,
                  physics: AlwaysScrollableScrollPhysics(),
                  children: [
                    ListView.builder(
                      physics: AlwaysScrollableScrollPhysics(),
                      padding: const EdgeInsets.all(0.0),
                      itemCount: todoList.keys.length,
                      itemBuilder: (context, index) {
                        final date = todoList.keys.elementAt(index);
                        return ExpansionTile(
                          initiallyExpanded: true,
                          leading: Icon(CupertinoIcons.calendar_today),
                          childrenPadding: const EdgeInsets.all(16.0),
                          title: Text(DateFormat(
                                  "EEEE, dd MMMM yyyy", "def.languageCode".tr)
                              .format(date)),
                          subtitle: Text(
                              "${todoList[date].length} task${"def.languageCode".tr == "en" && todoList[date].length > 1 ? "s" : ""}"),
                          children: [
                            for (var item in todoList[date])
                              TodoItem(item: item),
                          ],
                        );
                      },
                    ),
                    ListView.builder(
                      physics: AlwaysScrollableScrollPhysics(),
                      itemCount: list.outstandingList?.length ?? 0,
                      itemBuilder: (context, index) {
                        final item = list.outstandingList.elementAt(index);
                        return ListTile(
                          title: Text(item.masterProcessName),
                        );
                      },
                    ),
                  ],
                ),
              ),
            ),
          ],
        );
      },
    );
  }

  Map<DateTime, List<Activity>> getActivities(NotificationList list) {
    final todoList = list.todoList;
    if (todoList == null) return {};

    Map<DateTime, List<Activity>> result = {};
    for (var item in todoList) {
      if (item.idProcessInstance.toString().contains(filter.toLowerCase()) ||
          item.processName.toLowerCase().contains(filter.toLowerCase()) ||
          item.processCreatedByName
              .toLowerCase()
              .contains(filter.toLowerCase())) {
        final date = DateTime(
          item.processDateCreated.toLocal().year,
          item.processDateCreated.toLocal().month,
          item.processDateCreated.toLocal().day,
        );

        List<Activity> items;
        if (result.containsKey(date)) {
          items = result[date];
        } else {
          items = [];
          result[date] = items;
        }

        items.add(item);
      }
    }

    return result;
  }
}

class TodoItem extends StatelessWidget {
  const TodoItem({
    Key key,
    @required this.item,
  }) : super(key: key);

  final Activity item;

  @override
  Widget build(BuildContext context) {
    return Card(
      elevation: 8.0,
      child: InkWell(
        onTap: () {
          Navigator.push(
            context,
            MaterialPageRoute(
              builder: (context) => ProcessPage(
                idProcessInstance: item.idProcessInstance,
                idActivityNotification: item.id,
              ),
            ),
          );
        },
        child: IntrinsicHeight(
          child: Row(
            crossAxisAlignment: CrossAxisAlignment.start,
            children: [
              Container(
                width: 120,
                padding: const EdgeInsets.all(8.0),
                decoration: BoxDecoration(
                  borderRadius: BorderRadius.circular(16.0),
                  color: Theme.of(context).primaryColor,
                ),
                alignment: Alignment.center,
                child: DefaultTextStyle(
                  style: DefaultTextStyle.of(context).style.copyWith(
                        color: Colors.white,
                      ),
                  child: Column(
                    mainAxisSize: MainAxisSize.min,
                    children: [
                      Text(
                        DateFormat("HH:mm")
                            .format(item.processDateCreated.toLocal()),
                        style: TextStyle(
                          fontSize: 20.0,
                          fontWeight: FontWeight.bold,
                        ),
                      ),
                      Text(
                        item.processCreatedByName,
                        maxLines: 1,
                        overflow: TextOverflow.ellipsis,
                        style: TextStyle(
                          fontSize: 12.0,
                        ),
                      ),
                      Text("#${item.idProcessInstance}"),
                    ],
                  ),
                ),
              ),
              Expanded(
                child: Padding(
                  padding: const EdgeInsets.all(8.0),
                  child: Column(
                    crossAxisAlignment: CrossAxisAlignment.stretch,
                    children: [
                      Text(
                        item.processName,
                        style: TextStyle(
                          fontWeight: FontWeight.bold,
                        ),
                      ),
                      Text(
                        item.activityName,
                        style: TextStyle(
                          fontSize: 12.0,
                        ),
                      ),
                      Spacer(),
                      if (!item.isViewed)
                        Align(
                          alignment: Alignment.bottomRight,
                          child: Container(
                            width: 8,
                            height: 8,
                            decoration: BoxDecoration(
                              color: Theme.of(context).accentColor,
                              shape: BoxShape.circle,
                            ),
                          ),
                        ),
                    ],
                  ),
                ),
              ),
            ],
          ),
        ),
      ),
    );
  }
}
